'' =================================================================================================
''
''   File....... ehf_dc27_backpack.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2011-2019 Jon McPhalen
''   E-mail..... 
''   Started.... 
''   Updated.... 09 AUG 2019
''
'' =================================================================================================


con

  PCB_REV = "F"


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  TRM_BAUD = 115_200

  BTN_MS   = 25                                                 ' button debounce timing
  BTN_HOLD = 1000                                               ' button hold-off timing
                                                        
                                                                 
con { io pins }                                                  
                                                                 
  RX1     = 31  { I }                                           ' serial / programming
  TX1     = 30  { O }                                            
                                                                 
  SDA     = 29  { I/O }                                         ' i2c / eeprom
  SCL     = 28  { I/O }                                          
                                                                 
  XP0     = 27  { I/O }                                         ' extra port 0 - Rev F
  XP1     = 24  { I/O }                                         ' extra port 1 - Rev F
  XP2     = 19  { I/O }                                         ' extra port 2 - Rev F  
  XP3     = 16  { I/O }                                         ' extra port 3 - Rev F

  OUT_EN  = 27  { O }                                           ' outputs /enable - Rev E & lower     
  
  SIO     = 26  { I/O }                                         ' TTL serial io - Rev E & lower
                                                          
  SIO_RX  = 26  { I/O }                                         ' TTL serial io - Rev F   
  SIO_TX  = 25  { I/O }                                         ' TTL serial io - Rev F  (not enabled)  
                                                                 
  RX2     = 23  { I }                                           ' RS-485 port
  RX2_EN  = 22  { O }                                                
  TX2_EN  = 21  { O }                                                
  TX2     = 20  { O }                                                
  
  G_LED   = 18  { O }                                           ' bi-color LED (Rev F assignments)
  R_LED   = 17  { O }                                              
                                                                 
  OUT7    = 15  { O }                                           ' outputs
  OUT6    = 14  { O }                                                
  OUT5    = 13  { O }                                                
  OUT4    = 12  { O }                                                
  OUT3    = 11  { O }                                                
  OUT2    = 10  { O }                                                
  OUT1    =  9  { O }                                                
  OUT0    =  8  { O }                                                
                                                                 
  OPT_BR  =  7  { I/O }                                         ' options switches
  OPT_A1  =  6  { I/O }                                                 
  OPT_A0  =  5  { I/O }                                                 
  OPT_SM  =  4  { I/O }                                                 
                                                                 
  DMX_A8  =  3  { I }                                           ' high-bit of DMX address
                                                                 
  DO_165  =  2  { I }                                           ' control lines for 74x165
  CLK_165 =  1  { O }                                                 
  LD_165  =  0  { O }


con

  LEDZ    = OUT7
  SPKR    = OUT0                                              

                                           
con                                                              
                                                                 
  #(-1), ON, OFF' digital control
                                                                 
  #0, NO, YES                                                    
                                                                 
  #0, BLK, RED, GRN, YEL                                        ' for r/g led
                                                                 
  #0, NOCON, RX, TX, RXTX                                       ' for rs-485
                                                                 
  #0, LSBFIRST, MSBFIRST

  
  STRIP_LEN =  24
  INDOOR    = $20
  OUTDOOR   = $FF                                    
                                                                 
                                                                 
obj                              
                                 
' main                                                          ' * master Spin cog                   
  time       : "jm_time_80"                                     '   timing and delays  
  prng       : "jm_prng"                                        '   pseudo-random number generator     
  io         : "jm_io"                                          '   essential io
  ee         : "jm_24xx512"                                     '   eeprom
  strip      : "jm_rgbx_pixel"                                  ' * smart pixel control
  term       : "jm_fullduplexserial"                            ' * serial IO for terminal
' background                                                    ' * background RGY LED control, inputs scanning 
  
' * uses cog when loaded


dat

  PgmState      byte    0
  Bright        byte    1

  ' fix rotated LED ring

  ChMap       byte      20, 21, 22, 23,  0,  1,  2,  3
              byte       4,  5,  6,  7,  8,  9, 10, 11
              byte      12, 13, 14, 15, 16, 17, 18, 19   

                        
var

  long  pixbuf[STRIP_LEN]

                                                      
pub main | b                                                
                                                                 
  setup                                                         ' setup io and objects

  play_music(@Charge, 200)
  power_up
  repeat while (playing)
  time.pause(500)        
  
  repeat
    case pgmState
      0 : rolling_rainbow
      1 : alien_eye
      2 : pride_flag    


pub power_up | level, ch

  strip.clear
  time.pause(100)

  if (Bright == 0)
    level := INDOOR
  else
    level := OUTDOOR

  repeat ch from 0 to 11                                ' two red segments
    strip.set(ChMap[ch+00], level << 24)
    strip.set(ChMap[ch+12], level << 24)
    time.pause(50)

  strip.clear
  time.pause(100)

  repeat ch from 0 to 7                                 ' three green segments
    strip.set(ChMap[ch+00], level << 16)     
    strip.set(ChMap[ch+08], level << 16)    
    strip.set(ChMap[ch+16], level << 16)    
    time.pause(50)

  strip.clear
  time.pause(100)

  repeat ch from 0 to 5                                 ' four blue segments
    strip.set(ChMap[ch+00], level << 8)    
    strip.set(ChMap[ch+06], level << 8)   
    strip.set(ChMap[ch+12], level << 8)   
    strip.set(ChMap[ch+18], level << 8)       
    time.pause(50)

  strip.clear  


pub rolling_rainbow : phase | t, ch, b, level

  repeat
    if (Bright == 0)
      level := INDOOR
    else
      level := OUTDOOR
    repeat ch from 0 to STRIP_LEN-1
      strip.set(ch, strip.wheelx(256 / STRIP_LEN * ch + phase, level))
    t := cnt
    if (check_button == 0)  
      waitcnt(t += constant(MS_001 * 3))
    if (pgmState <> 0)
      return
    phase := ++phase & $FF


pub alien_eye | level

'' from 1953 War of the Worlds

  repeat
    if (Bright == 0)
      level := INDOOR
    else
      level := OUTDOOR

    ' fixed for rotation of ring

    strip.fill(20, 23, level <<  8)                             ' blue
    strip.fill( 0,  3, level <<  8)
    strip.fill( 4, 11, level << 24)                             ' red    
    strip.fill(12, 19, level << 16)                             ' green

    check_button
    if (pgmState <> 1)
      return      

    
pub pride_flag | level

  repeat
    if (Bright == 0)
      level := INDOOR
    else
      level := OUTDOOR

    ' fixed for rotation of ring

    strip.fill( 4,  7, strip.scale_rgbw(strip#RED,    level))  
    strip.fill( 8, 11, strip.scale_rgbw(strip#ORANGE, level))
    strip.fill(12, 15, strip.scale_rgbw(strip#YELLOW, level))    
    strip.fill(16, 19, strip.scale_rgbw(strip#GREEN,  level))
    strip.fill(20, 23, strip.scale_rgbw(strip#BLUE,   level))
    strip.fill( 0,  3, strip.scale_rgbw(strip#PURPLE, level))         
                                        
    check_button
    if (pgmState <> 2)
      return   


pub back_off_jack | level

  tone_on(100_0, 3000)                                  ' groan

  repeat 10                                             ' flash red
    strip.set_all(strip#RED)
    time.pause(50)
    strip.clear
    time.pause(50) 
    
  repeat level from 255 to 0                            ' nice fade out
    strip.set_all(strip.gamma(level) << 24)
    time.pause(4)

  strip.clear
  note_off


pub check_button | b

  b := button_status
  case b
    1 :
      play_music(@Vader, 120)    

    2 :
      play_music(@StarWars, 200)

    3 :
      play_music(@Charge, 240)

    4 :
      if (PgmState <> 0)
        PgmState := 0
        play_music(@Beep, 240)
        ee.wr_byte(@PgmState, PgmState)   

    5 :
      if (PgmState <> 1)
        PgmState := 1
        play_music(@Beep, 240)
        ee.wr_byte(@PgmState, PgmState)

    6 :
      if (PgmState <> 2)
        PgmState := 2
        play_music(@Beep, 240)
        ee.wr_byte(@PgmState, PgmState)

    100 :                                                       ' anti-creeper mode
      back_off_jack
      return 1

    300:                                                        ' change led brightness
      if (Bright)
        Bright := 0
        play_music(@Beeps0, 240) 
      else
        Bright := 1
        play_music(@Beeps1, 240)
      ee.wr_byte(@Bright, Bright)         
        
  
                                                                        
pub setup
                                                                 
'' Configure HC-8+ IO and support objects                        
                                                                 
  time.start                                                    ' setup for timing/delays

  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed prng
                                                                 
  io.start(0, 0)                                                ' clear pins (this cog)

  ee.start(%000)                                                ' connect to EE

  strip.start_2812b(@pixbuf, STRIP_LEN, LEDZ, 0_1)              ' connect to WS2812s

  term.start(RX1, TX1, %0000, TRM_BAUD)                         ' start serial for terminal *

  bcog := cognew(background, @bcstack) + 1                      ' start background cog *
                                                    

con

  { --------------------------------- }
  {  S U P P O R T   R O U T I N E S  } 
  { --------------------------------- }


pub ttl_pin                                                      
                                                                 
'' Reads TTL states for 1-of-8 input                             
'' -- returns -1 (no input)                                      
'' -- returns 0 to 7 for input on IN0 to IN7                     
''    * only one input allowed                                   
                                                                 
  if (bit_count(ttlpins) == 1)                                  ' only one button pressed?
    return bit_pos(ttlpins, 0)                                  '  yes, return button #
  else                                                           
    return -1                                                   '  none or bad input
                                                                 

pub ttl_inputs 

'' Return TTL inputs status
'' -- pins are scanned by background cog

  return ttlpins


pub dmx_address 

'' Return DMX address switch setting
'' -- switches are scanned by background cog
'' -- Note: For Rev C and lower boards, do not use DMX 256 switch -- leave it OFF

  return dmxaddr


pub option_inputs

'' Returns state of option switch bits (%0000..%1111, all off to all on)

  dira[OPT_BR..OPT_SM] := %0000                                 ' force to input mode
                                                                  
  return !ina[OPT_BR..OPT_SM] & $0F                             ' return state (1 = on)
                                                                  

pub xport_bits

'' Returns state of xport bits (%0000..%1111, all off to all on)
'' -- does not set dira bits to 0

  if (PCB_REV < "F")
    return %0000
  else
    return (ina[XP3] << 3) | (ina[XP2] << 2) | (ina[XP1] << 1) | ina[XP0]   

                      
pub set_rs485(rxtxstate)                                              
                                                                  
'' Sets RS-485 RX and TX enable inputs                            
                                                                  
  case rxtxstate                                                      
    NOCON : outa[RX2_EN..TX2_EN] := %10                           
    RX    : outa[RX2_EN..TX2_EN] := %00                           
    TX    : outa[RX2_EN..TX2_EN] := %11                           
    RXTX  : outa[RX2_EN..TX2_EN] := %01                           
                                                                  
  dira[RX2_EN..TX2_EN] := %11                                     


pub bit_count(value) : bc

'' Returns # of bits set (1) in value 

  repeat while (value)
    value &= value-1
    ++bc                                                     
                                                                 
                                                                 
pub bit_pos(value, mode)                                         
                                                                 
'' Returns position of 1st bit set (0..31)                       
'' -- mode 0 to scan from lsb, mode 1 to scan from msb           
'' -- -1 = no bits set                                           
                                                                 
  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1
  else                                                           
    if (mode == LSBFIRST)                                       ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)                                      
    else                                                          
      return (>|value - 1)                                        
                                                                  
                                                                  
pub bit_val(value, pos)                                           
                                                            
'' Returns bit (0..1) from pos (0..31) in value                   
                                                                  
  if ((pos => 0) and (pos =< 31))                                 
    return (value >> pos) & 1                                     
  else                                                            
    return 0


con

  { ----------------------------- }
  {  B A C K G R O U N D   C O G  }
  {  - global milliseconds        }
  {  - R/G LED                    }
  {  - TTL/DMX inputs scanning    }
  {  - TTL input debounce timing  }
  { ----------------------------- }


pub button_status

'' Returns button clicks if any available since last check

  if (btnbusy == 0)                                             ' okay to read?
    result := btnclix                                           ' return clicks
    btnclix := 0
  else
    result := 0


pub play_music(p_music, rate)

'' Play musical notes at p_music
'' -- rate is beats per minute

  BPM     := rate
  p_notes := p_music


pub tone_on(freq, ms)

'' Play tone of freq for ms milliseconds
'' -- non-blocking
'' -- freq in 0.1Hz units

  note_off
  time.pause(1)
  longmove(@OneNote, @freq, 2)
  play_music(@OneNote, 60)


pub note_off

'' Stops note or music if playing

  longfill(@p_notes, -1, 2)


pub playing

'' Returns true when note or music is playing

  return (p_notes > 0) 


var

  long  bcog                                                    ' background cog #
  long  bcstack[32]                                             ' stack for background cog
                                                                 
  long  millis                                                  ' global milliseconds register
                                                                 
                                                                 
pri background | t                                              ' launch with cognew()
                                                                 
  io.high(LD_165)                                               ' setup x165 io pins
  io.low(CLK_165)                                                
  io.input(DO_165)                                               
                                                                 
  millis := 0                                                    
                                                                 
  t := cnt                                                      ' sync loop timer
  repeat
    waitcnt(t += MS_001)                                        ' run loop every millisecond
    ++millis                                                     
    scan_ttl_ins                                                ' re-scan TTL & DMX inputs
    scan_button                                                 ' scan pgm control button
    play_song                                                   ' play musical notes when enabled
                                                          

var

  long  ttlpins
  long  dmxaddr                                                  

                                                                
pri scan_ttl_ins | tempin                                         
                                                                 
'' Scan TTL and DMX address inputs on stacked x165s                              
                                                                 
  outa[LD_165] := 0                                             ' blip Shift/Load line    
  outa[LD_165] := 1                                              

  ' unrolled for best speed

  tempin := ina[DMX_A8]                                         ' bit16

  tempin := (tempin << 1) | ina[DO_165]                         ' bit15
  outa[CLK_165] := 1                                            ' blip clock  
  outa[CLK_165] := 0                                                        
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0                   
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0                   
  tempin := (tempin << 1) | ina[DO_165]
  outa[CLK_165] := 1                   
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  
  tempin := (tempin << 1) | ina[DO_165]                         ' bit7
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                    
  tempin := (tempin << 1) | ina[DO_165] 
  outa[CLK_165] := 1                    
  outa[CLK_165] := 0                                       
  tempin := (tempin << 1) | ina[DO_165]                         ' bit0                    
                                                         
  ttlpins := tempin.byte[0]                                     ' update global vars
  dmxaddr := tempin >> 8    


var

  long  btnstate
  long  btntimer
  long  btnbusy
  long  btnclix


pri scan_button

'' Scan button and count discrete clicks

  btnstate := ((btnstate << 1) & %11) | (ttlpins & %1)          ' update edge state

  case btnstate
    %01:                                                        ' new press  
      btntimer := 0                                                      
      btnbusy := 1
       
    %11:                                                        ' hold
      ++btntimer

    %10:                                                        ' new release
      case btntimer
        50..350 :
          ++btnclix
        1000..2000:
          btnclix := 100
          btnbusy := 0
        3000..5000:
          btnclix := 300
          btnbusy := 0
        other:
          btnbusy := 0
      btntimer := 0

    %00:                                                        ' off time
      if (btnbusy) 
        if (--btntimer < -500)                                  ' past inter-press threshold
          btnbusy := 0


var

  long  p_notes
  long  nmillis


dat

  BPM           long    120
  
  OneNote       long    0 {freq}, 0 {ms}, -1, -1
                         
  Charge        long    N_G3, T_8TH,         N_C4, T_8TH, N_E4, T_8TH
                long    N_G4, T_8TH * 3 / 2, N_E4, T_8TH, N_G4, T_DHALF
                long    0, 0, -1, -1

  Beeps1        long    N_C5, T_16TH, REST, T_16TH
                long    N_C5, T_16TH, REST, T_16TH 
  Beep          long    N_C5, T_16TH, REST, T_8TH  
                long    0, 0, -1, -1

  Beeps0        long    N_C3, T_16TH, REST, T_16TH
                long    N_C3, T_16TH, REST, T_16TH 
                long    N_C3, T_16TH, REST, T_8TH  
                long    0, 0, -1, -1                 
                         
                         
  StarWars      long    N_D3, T_8TH, REST, T_64TH
                long    N_D3, T_8TH, REST, T_64TH
                long    N_D3, T_8TH, REST, T_64TH  
                long    N_G3, T_HALF
                long    N_D4, T_HALF
                long    N_C4, T_8TH, REST, T_64TH
                long    N_B3, T_8TH, REST, T_64TH
                long    N_A3, T_8TH, REST, T_64TH
                long    N_G4, T_HALF
                long    N_D4, T_4TH, REST, T_16TH
                long    N_C4, T_8TH, REST, T_64TH
                long    N_B3, T_8TH, REST, T_64TH
                long    N_A3, T_8TH, REST, T_64TH
                long    N_G4, T_HALF
                long    N_D4, T_4TH, REST, T_16TH      
                long    N_C4, T_8TH, REST, T_64TH
                long    N_B3, T_8TH, REST, T_64TH
                long    N_C4, T_8TH, REST, T_64TH   
                long    N_A3, T_HALF
                long    0, 0, -1, -1


  ' https://www.musicnotes.com/sheetmusic/mtd.asp?ppn=MN0133739

  Vader         long    N_A3,  T_4TH,  REST, T_64TH
                long    N_A3,  T_4TH,  REST, T_64TH
                long    N_A3,  T_4TH,  REST, T_64TH
                long    N_F3,  T_8TH,  REST, T_64TH
                long    N_C4,  T_8TH,  REST, T_64TH
                long    N_A3,  T_4TH,  REST, T_64TH
                long    N_F3,  T_8TH,  REST, T_64TH
                long    N_C4,  T_8TH,  REST, T_64TH
                long    N_A3,  T_HALF, REST, T_64TH
                long    N_E4,  T_4TH,  REST, T_64TH  
                long    N_E4,  T_4TH,  REST, T_64TH  
                long    N_E4,  T_4TH,  REST, T_64TH  
                long    N_F4,  T_8TH,  REST, T_64TH  
                long    N_C4,  T_8TH,  REST, T_64TH
                long    N_Ab3, T_4TH,  REST, T_64TH  
                long    N_F3,  T_8TH,  REST, T_64TH
                long    N_C4,  T_8TH,  REST, T_64TH
                long    N_A3,  T_HALF, REST, T_64TH
                long    0, 0, -1, -1                                
         

pri play_song | nfreq

  if (p_notes < 0)                                              ' no notes to play?
    ctra := 0
    outa[SPKR] := 0
    return

  if (--nmillis > 0)                                            ' is note playing now?
    return

  nfreq := long[p_notes]
  p_notes += 4        
  if (nfreq < 0)
    ctra := 0 
    longfill(@p_notes, -1, 2)                                   ' end of song, kill
    outa[SPKR] := 0
    return

  nmillis := (0 #> long[p_notes]) * 60 / BPM
  p_notes += 4                       

  ctra := constant(((%00100) << 26) | SPKR) 
  frqa := ($8000_0000 / (constant(CLK_FREQ * 10) / nfreq)) << 1  
  dira[SPKR] := 1

  
con

  ' base timing at 60 BPM
  
  T_WHOLE = 4000
  T_DHALF = T_WHOLE * 3 / 4                                  
  T_HALF  = T_WHOLE >> 1
  T_4TH   = T_WHOLE >> 2
  T_8TH   = T_WHOLE >> 3
  T_16TH  = T_WHOLE >> 4
  T_32ND  = T_WHOLE >> 5
  T_64TH  = T_WHOLE >> 6    

  ' frequencies in 0.1Hz units

  REST    =    0_0

  N_C0    =   16_3 
  N_Cs0   =   17_3                                              ' sharp
  N_Db0   =   17_3                                              ' flat
  N_D0    =   18_3 
  N_Ds0   =   19_4 
  N_Eb0   =   19_4 
  N_E0    =   20_6 
  N_F0    =   21_8 
  N_Fs0   =   23_1
  N_Gb0   =   23_1  
  N_G0    =   24_5 
  N_Gs0   =   25_9
  N_Ab0   =   25_9  
  N_A0    =   27_5 
  N_As0   =   29_1
  N_Bb0   =   29_1
  N_B0    =   30_8 

  N_C1    =   32_7
  N_Cs1   =   34_6
  N_Db1   =   34_6  
  N_D1    =   36_7
  N_Ds1   =   38_8
  N_Eb1   =   38_8 
  N_E1    =   41_2
  N_F1    =   43_6
  N_Fs1   =   46_2
  N_Gb1   =   46_2  
  N_G1    =   49_0
  N_Gs1   =   51_9
  N_Ab1   =   51_9  
  N_A1    =   55_0
  N_As1   =   58_2
  N_Bb1   =   58_2   
  N_B1    =   61_7
  
  N_C2    =   65_4 
  N_Cs2   =   69_3
  N_Db2   =   69_3  
  N_D2    =   73_4 
  N_Ds2   =   77_8
  N_Eb2   =   77_8 
  N_E2    =   82_4 
  N_F2    =   87_3 
  N_Fs2   =   92_5
  N_Gb2   =   92_5  
  N_G2    =   98_0 
  N_Gs2   =  103_8
  N_Ab2   =  103_8 
  N_A2    =  110_0 
  N_As2   =  116_5
  N_Bb2   =  116_5   
  N_B2    =  123_5 

  N_C3    =  130_8
  N_Cs3   =  138_6
  N_Db3   =  138_6  
  N_D3    =  146_8
  N_Ds3   =  155_6
  N_Eb3   =  155_6 
  N_E3    =  164_8
  N_F3    =  174_6
  N_Fs3   =  185_0
  N_Gb3   =  185_0 
  N_G3    =  196_0
  N_Gs3   =  207_7
  N_Ab3   =  207_7  
  N_A3    =  220_0
  N_As3   =  233_0
  N_Bb3   =  233_0  
  N_B3    =  247_0

  N_C4    =  261_6 
  N_Cs4   =  277_2
  N_Db4   =  277_2    
  N_D4    =  293_7 
  N_Ds4   =  311_1
  N_Eb4   =  311_1  
  N_E4    =  329_6 
  N_F4    =  349_2 
  N_Fs4   =  370_0
  N_Gb4   =  370_0    
  N_G4    =  392_0 
  N_Gs4   =  415_3
  N_Ab4   =  415_3     
  N_A4    =  440_0 
  N_As4   =  466_1
  N_Bb4   =  466_1  
  N_B4    =  493_9 

  N_C5    =  523_3    
  N_Cs5   =  554_4
  N_Db5   =  554_4     
  N_D5    =  587_3    
  N_Ds5   =  622_2
  N_Eb5   =  622_2     
  N_E5    =  659_2    
  N_F5    =  698_5    
  N_Fs5   =  740_0
  N_Gb5   =  740_0    
  N_G5    =  784_0    
  N_Gs5   =  830_6
  N_Ab5   =  830_6     
  N_A5    =  880_0    
  N_As5   =  932_3
  N_Bb5   =  932_3    
  N_B5    =  987_8    

  N_C6    = 1046_5   
  N_Cs6   = 1108_7
  N_Db6   = 1108_7   
  N_D6    = 1174_7   
  N_Ds6   = 1244_5
  N_Eb6   = 1244_5   
  N_E6    = 1318_5   
  N_F6    = 1396_9   
  N_Fs6   = 1480_0
  N_Gb6   = 1480_0     
  N_G6    = 1568_0   
  N_Gs6   = 1661_2
  N_Ab6   = 1661_2    
  N_A6    = 1760_0   
  N_As6   = 1864_7
  N_Bb6   = 1864_7    
  N_B6    = 1975_5   
  
  N_C7    = 2093_0     
  N_Cs7   = 2217_5
  N_Db7   = 2217_5       
  N_D7    = 2349_3     
  N_Ds7   = 2489_0
  N_Eb7   = 2489_0       
  N_E7    = 2637_0     
  N_F7    = 2793_8     
  N_Fs7   = 2960_0
  N_Gb7   = 2960_0       
  N_G7    = 3136_0     
  N_Gs7   = 3322_4
  N_Ab7   = 3322_4      
  N_A7    = 3520_0     
  N_As7   = 3729_3
  N_Bb7   = 3729_3      
  N_B7    = 3951_0     

  N_C8    = 4186_0    
  N_Cs8   = 4434_9
  N_Db8   = 4434_9   
  N_D8    = 4698_6    
  N_Ds8   = 4978_0
  N_Eb8   = 4978_0      
  N_E8    = 5274_0    
  N_F8    = 5587_7    
  N_Fs8   = 5919_9
  N_Gb8   = 5919_9     
  N_G8    = 6271_9    
  N_Gs8   = 6644_9
  N_Ab8   = 6644_9     
  N_A8    = 7040_0    
  N_As8   = 7458_6
  N_Bb8   = 7458_6      
  N_B8    = 7902_1    
                            
      
dat { license }                                                  

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      